@@ -250,22 +250,28 @@ class Session(object):
         self._managed_request = None
         self._request = requests.Session()
         
-        if cookies is not None:
+        if cookies:
             if not isinstance(cookies, collections_abc.Mapping):
                 raise TypeError('The cookies argument is required to be a mapping.')
             self._request.cookies.update(cookies)
-        
-        if headers is not None:
+
+        if headers:
             if not isinstance(headers, collections_abc.Mapping):
                 raise TypeError('The headers argument is required to be a mapping.')
-            self._request.headers.update(headers)
+
+            headers = dict(headers)
+
+        else:
+            headers = {}
         
         if not isinstance(strict_api, bool):
             raise TypeError('The strict_api argument is required to be a boolean.')
-        self._request.headers.update(
+
+        headers.update(
             {'ftrack-strict-api': 'true' if strict_api is True else 'false'}
         )
-        
+
+        self._request.headers.update(headers)
         self._request.auth = SessionAuthentication(
             self._api_key, self._api_user
         )
@@ -287,7 +293,7 @@ class Session(object):
             self._server_url,
             self._api_user,
             self._api_key,
-            headers=self._request.headers,
+            headers=headers,
             cookies=requests.utils.dict_from_cookiejar(self._request.cookies)
         )
 
@@ -1686,26 +1692,33 @@ class Session(object):
 
         self.logger.debug(L('Calling server {0} with {1!r}', url, data))
 
-        response = self._request.post(
-            url,
-            headers=headers,
-            data=data,
-            timeout=self.request_timeout,
-        )
+        try:
+            response = self._request.post(
+                url,
+                headers=headers,
+                data=data,
+                timeout=self.request_timeout,
+            )
+            response.raise_for_status()
+
+        except requests.exceptions.HTTPError:
+            error_message = (
+                'Server reported error: {0}'
+                .format(response.text)
+            )
+            self._raise_server_error(error_message)
 
         self.logger.debug(L('Call took: {0}', response.elapsed.total_seconds()))
-
         self.logger.debug(L('Response: {0!r}', response.text))
-        try:
-            result = self.decode(response.text)
 
+        try:        
+            result = self.decode(response.text)
         except Exception:
             error_message = (
                 'Server reported error in unexpected format. Raw error was: {0}'
                 .format(response.text)
             )
-            self.logger.exception(error_message)
-            raise ftrack_api.exception.ServerError(error_message)
+            self._raise_server_error(error_message)
 
         else:
             if 'exception' in result:
@@ -1713,11 +1726,13 @@ class Session(object):
                 error_message = 'Server reported error: {0}({1})'.format(
                     result['exception'], result['content']
                 )
-                self.logger.exception(error_message)
-                raise ftrack_api.exception.ServerError(error_message)
-
+                self._raise_server_error(error_message)
         return result
 
+    def _raise_server_error(self, error_message):
+        self.logger.exception(error_message)
+        raise ftrack_api.exception.ServerError(error_message)
+
     def encode(self, data, entity_attribute_strategy='set_only'):
         '''Return *data* encoded as JSON formatted string.
 
